(() => {
	if (document.getElementById("awmt-overlay")) return false;

	const overlay = document.createElement("div");
	overlay.setAttribute("id", "awmt-overlay");

	const overlayInner = document.createElement("div");
	overlayInner.className = "awmt-overlay-inner";

	const overlayIcon = document.createElement("div");
	overlayIcon.className = "awmt-overlay-icon";

	const overlayImage = document.createElement("img");
	overlayImage.src = chrome.runtime.getURL("icons/plus-black.svg");
	overlayImage.className = "awmt-overlay-image";

	const overlayImageHover = document.createElement("img");
	overlayImageHover.src = chrome.runtime.getURL("icons/plus-white.svg");
	overlayImageHover.className = "awmt-overlay-image-hover";

	const overlayText = document.createElement("div");
	overlayText.innerText = "Save to my mind";
	overlayText.className = "awmt-overlay-text";

	overlayIcon.appendChild(overlayImage);
	overlayIcon.appendChild(overlayImageHover);
	overlayInner.appendChild(overlayText);
	overlay.appendChild(overlayIcon);
	overlay.appendChild(overlayInner);

	document.body.appendChild(overlay);

	window.setTimeout(() => {
		const overlayWidth = overlay.clientWidth;
		const overlayHeight = overlay.clientHeight;

		document.addEventListener("mouseenter", event => {
			const target = event.target;

			if (target.tagName != "DIV" || !target.parentElement) return;

			if (target.parentElement.tagName == "A") {
				target.classList.add("awmt-instagram-overlay-target");
			}

			const item = target.parentElement.querySelector(":scope > div > img, :scope > div > div > img");

			if (!item || item.classList.contains("awmt-overlay-image")) return;

			const width = item.clientWidth;
			const height = item.clientHeight;

			if (width < 200 || height < 100) return;

			const src = item.currentSrc || item.src;

			if (src) {
				const offset = getOffset(item);
				const position = {
					top: offset.top + height - overlayHeight - 20,
					left: offset.left + width - overlayWidth - 20,
				}

				const url = getUrl(item);
				const user = getUser(item);

				overlay.setAttribute("data-src", src);
				overlay.setAttribute("data-url", url || "");
				overlay.setAttribute("data-user", user || "");

				overlay.style.top = `${position.top}px`;
				overlay.style.left = `${position.left}px`;

				showOverlay();
			}
		}, true);

		document.addEventListener("mouseleave", event => {
			const target = event.target;

			if (target.tagName != "DIV" || !target.parentElement) return;

			const item = target.parentElement.querySelector(":scope > div > img, :scope > div > div > img");

			if (!item || item.classList.contains("awmt-overlay-image")) return;

			hideOverlay();
		}, true);
	}, 0);

	overlay.addEventListener("click", event => {
		const src = overlay.getAttribute("data-src");
		const url = overlay.getAttribute("data-url");
		const user = overlay.getAttribute("data-user");

		const config = {
			saveImage: src,
			data: {
				pageUrl: url || undefined,
				pageMeta: {
					username: user || undefined
				}
			}
		};

		if (src) {
			chrome.runtime.sendMessage(config);
			hideOverlay();
		}
	});

	function getOffset(item) {
		const rect = item.getBoundingClientRect();
		const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
		const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
		return { top: rect.top + scrollTop, left: rect.left + scrollLeft };
	}

	function getUrl(item) {
		const parentLink = item.closest("a");

		if (parentLink) return parentLink.href;

		const parentWrap = item.closest("article");

		if (!parentWrap) return;

		const links = parentWrap.querySelectorAll("a");
		let postUrl;

		if (links.length) {
			links.forEach(link => {
				const url = link.href;
				if (url && url.includes("/p/")) postUrl = url;
			});
		}

		if (postUrl) return postUrl;
	}

	function getUser(item) {
		const nameWrap = document.querySelector("main header h2");

		if (nameWrap) return nameWrap.innerText;

		const parentWrap = item.closest("article");

		if (!parentWrap) return;

		const userLink = parentWrap.querySelector("header a");

		if (!userLink) return;

		const urlParts = userLink.href.split('/');

		if (urlParts.length > 1) return urlParts[urlParts.length - 2];
	}

	function showOverlay() {
		overlay.setAttribute("open", true);
	}

	function showOverlay() {
		overlay.setAttribute("open", true);
	}

	function hideOverlay() {
		overlay.setAttribute("open", false);
	}
})();
