(() => {
	const usePromise = typeof browser !== "undefined";

	const events = {};
	const { id, icon, item, message, status, redirect } = window.awmt;

	const createListener = (id, target, name, callback) => {
		events[id] = {
			target,
			name,
			callback: target.addEventListener(name, callback)
		}
	};

	const notification = document.getElementById("awmt-notification");

	notification.removeAttribute("closing");
	if (status && status !== "undefined") notification.setAttribute("status", status);

	const notificationWrap = document.createElement("div");
	notificationWrap.className = "awmt-notification-wrap";

	const notificationInner = document.createElement("div");
	notificationInner.className = "awmt-notification-inner";

	const notificationIcon = document.createElement("img");
	notificationIcon.src = icon;

	const notificationText = document.createElement("span");
	notificationText.innerText = message;

	notificationInner.appendChild(notificationIcon);
	notificationInner.appendChild(notificationText);
	notificationWrap.appendChild(notificationInner);

	if (status === "success") {
		const notificationTags = document.createElement("div");
		notificationTags.className = "awmt-notification-tags";

		const notificationTagsStorage = document.createElement("div");
		notificationTagsStorage.className = "awmt-tagstorage";

		const notificationInput = document.createElement("input");
		notificationInput.type = "text";
		notificationInput.placeholder = "Add tags";

		createListener("keydown", notificationInput, "keydown", event => {
			const value = event.target.value.trim();
			if (value === "") return;
			if (event.key === "Enter") {
				function callback(response) {
					addTag(value, response);
				};

				const config = { addTag: event.target.value, item };

				if (usePromise) {
					browser.runtime.sendMessage(config).then(callback);
				} else {
					chrome.runtime.sendMessage(config, callback);
				}

				notificationInput.value = "";
			}
		});

		notificationTags.appendChild(notificationTagsStorage);
		notificationTags.appendChild(notificationInput);
		notificationWrap.appendChild(notificationTags);

		window.setTimeout(() => {
			notificationInput.focus();

			createListener("focus", notificationInput, "focus", event => {
				notification.setAttribute("focus", true);
			});
		}, 100);

		createListener("keydown", notificationInput, "keydown", event => {
			notification.setAttribute("focus", true);
		});

		createListener("blur", notificationInput, "blur", event => {
			notification.removeAttribute("focus");
			createRemovalTimeout(4000);
		});

		const addTag = function(value) {
			const notificationTag = document.createElement("div");
			notificationTag.className = "awmt-notification-tag";
			notificationTag.innerText = value;
			notificationTagsStorage.appendChild(notificationTag);
		}.bind(this);
	}

	notification.innerHTML = "";
	notification.appendChild(notificationWrap);

	createListener("click", notificationInner, "click", event => {
		if (redirect && redirect !== "undefined") {
			chrome.runtime.sendMessage({ openTab: redirect });
		}

		createRemovalTimeout(0, true);
	});

	createListener("mouseleave", notification, "mouseleave", event => {
		notification.removeAttribute("mouse");
		createRemovalTimeout(4000);
	});

	createListener("mouseenter", notification, "mouseenter", event => {
		notification.setAttribute("mouse", true);
	});

	const createRemovalTimeout = (amt, forceClose) => {
		if (window.awmt.id !== id) {
			removeListeners();
			return false;
		}

		setTimeout(() => {
			const isActive = notification.getAttribute("focus") || notification.getAttribute("mouse");
			const isClosing = notification.getAttribute("closing");

			if (forceClose || (!isActive && !isClosing)) {
				notification.addEventListener("animationend", () => {
					if (window.awmt.id === id) notification.parentElement.removeChild(notification);
				});

				notification.setAttribute("closing", true);
			}
		}, amt);
	};

	const removeListeners = () => {
		Object.values(events).forEach(({ target, name, callback }) => target.removeEventListener(name, callback));
	};

	setTimeout(() => {
		createRemovalTimeout(0);
	}, 4000);
})();
