(() => {
	if (document.getElementById("awmt-overlay")) return false;

	const overlay = document.createElement("div");
	overlay.setAttribute("id", "awmt-overlay");

	const overlayInner = document.createElement("div");
	overlayInner.className = "awmt-overlay-inner";

	const overlayIcon = document.createElement("div");
	overlayIcon.className = "awmt-overlay-icon";

	const overlayImage = document.createElement("img");
	overlayImage.src = chrome.runtime.getURL("icons/plus-black.svg");
	overlayImage.className = "awmt-overlay-image";

	const overlayImageHover = document.createElement("img");
	overlayImageHover.src = chrome.runtime.getURL("icons/plus-white.svg");
	overlayImageHover.className = "awmt-overlay-image-hover";

	const overlayText = document.createElement("div");
	overlayText.innerText = "Save to my mind";
	overlayText.className = "awmt-overlay-text";

	overlayIcon.appendChild(overlayImage);
	overlayIcon.appendChild(overlayImageHover);
	overlayInner.appendChild(overlayText);
	overlay.appendChild(overlayIcon);
	overlay.appendChild(overlayInner);

	document.body.appendChild(overlay);

	window.setTimeout(() => {
		const overlayWidth = overlay.clientWidth;
		const overlayHeight = overlay.clientHeight;

		document.addEventListener("mouseenter", event => {
			const item = event.target;

			if (item.tagName != "IMG") return;

			const width = item.clientWidth;
			const height = item.clientHeight;

			if (width < 200 || height < 100) return;

			const src = item.currentSrc || item.src;

			if (src) {
				const offset = getOffset(item);
				const position = {
					top: offset.top + height - overlayHeight - 20,
					left: offset.left + width - overlayWidth - 20,
				}

				overlay.setAttribute("data-src", src);
				overlay.style.top = `${position.top}px`;
				overlay.style.left = `${position.left}px`;

				showOverlay();
			}
		}, true);

		document.addEventListener("mouseleave", event => {
			const item = event.target;

			if (item.tagName === "IMG") {
				hideOverlay();
			}
		}, true);
	}, 0);

	overlay.addEventListener("click", event => {
		const src = overlay.getAttribute("data-src");
		const config = { saveImage: src };

		if (src) {
			chrome.runtime.sendMessage(config);
			hideOverlay();
		}
	});

	function getOffset(item) {
		const rect = item.getBoundingClientRect();
		const scrollLeft = window.pageXOffset || document.documentElement.scrollLeft;
		const scrollTop = window.pageYOffset || document.documentElement.scrollTop;
		return { top: rect.top + scrollTop, left: rect.left + scrollLeft };
	}

	function showOverlay() {
		overlay.setAttribute("open", true);
	}

	function hideOverlay() {
		overlay.setAttribute("open", false);
	}
})();
