(() => {
	if (document.getElementById("awmt-prompt")) return false;

	const prompt = document.createElement("div");
	prompt.setAttribute("id", "awmt-prompt");

	const promptInner = document.createElement("div");
	promptInner.className = "awmt-prompt-inner";

	const promptIcon = document.createElement("img");
	promptIcon.src = chrome.runtime.getURL("icons/close.svg");
	promptIcon.className = "awmt-prompt-icon";

	const promptClose = document.createElement("div");
	promptClose.appendChild(promptIcon);
	promptClose.className = "awmt-prompt-close";

	const promptText = document.createElement("div");
	promptText.className = "awmt-prompt-text";

	promptInner.appendChild(promptText);
	promptInner.appendChild(promptClose);
	prompt.appendChild(promptInner);

	document.body.appendChild(prompt);

	let hideTimer;
	const listeners = {
		selectionchange: () => {
			if (!document.getSelection().isCollapsed) {
				promptText.innerText = "Remember this text highlight";
				prompt.setAttribute("event", "selectionchange");
				showPrompt();
			}
		}
	};

	Object.entries(listeners).forEach(([key, value]) => {
		document.addEventListener(key, value);
	});

	promptText.addEventListener("click", function() {
		const event = prompt.getAttribute("event");

		let config;

		if (event === "selectionchange") {
			const selection = window.getSelection().toString();
			config = { saveText: selection };
		}

		if (config) {
			chrome.runtime.sendMessage(config);
			hidePrompt();
		}
	});

	promptClose.addEventListener("click", function() {
		const event = prompt.getAttribute("event");
		document.removeEventListener(event, listeners[event]);
		hidePrompt(0);
	});

	document.addEventListener("selectionchange", function() {
		if (document.getSelection().isCollapsed && prompt.getAttribute("event") === "selectionchange") {
			hidePrompt();
		}
	});

	function showPrompt() {
		prompt.removeAttribute("closing");

		if (hideTimer) {
			window.clearTimeout(hideTimer);
		}

		if (prompt.getAttribute("open") === "true") {
			return;
		}

		prompt.setAttribute("open", true);
	}

	function hidePrompt(delay = 300) {
		if (prompt.getAttribute("closing") || prompt.getAttribute("open") === "false") {
			return;
		}

		prompt.setAttribute("closing", true);

		hideTimer = window.setTimeout(function() {
			prompt.removeAttribute("event");
			prompt.removeAttribute("closing");
			prompt.setAttribute("open", false);
		}, delay);
	}
})();
